==> schema31-safe-hub-lib-0.1.2-bb5686.zip: src/Application/Service/User.php <==
<?php

namespace Schema31\SafeHubLib\Application\Service;

use Schema31\SafeHubLib\IdentityProvider\SafeIdentityProvider;
use Schema31\SafeHubLib\IdentityProvider\GoogleIdentityProvider;
use Schema31\SafeHubLib\IdentityProvider\FacebookIdentityProvider;

use Schema31\SafeHubLib\Domain\Repository\PhysicalEntityRepositoryInterface;

/**
 * Description of User
 *
 * @author Antonio Turdo <aturdo@schema31.it>
 */
class User {
    
    private $identityProviders;
    private $physicalEntityRepository;
    
    public function __construct(PhysicalEntityRepositoryInterface $physicalEntityRepository) {
        $this->identityProviders = [
            new SafeIdentityProvider(),
            new GoogleIdentityProvider(),
            new FacebookIdentityProvider()
        ];
        
        $this->physicalEntityRepository = $physicalEntityRepository;
    }
    
    public function createUser(string $firstName, string $lastName, string $fiscalCode, string $email, \stdClass $identity, bool $createUserDB = false) {
		$identityProvider = $this->getProvider($identity);

        // validate the identity
        $identityProvider->validate($identity);        
        
        // normalize the identity
        $identityProvider->normalize($identity);
        
        $providerKey = $identityProvider->getProviderKey();
		
        // look for existing physical entities with the provided identity
        $userByIdentity = $this->physicalEntityRepository->findOneByIdentity($providerKey, $identity->$providerKey->Loginkey);
        
        if($userByIdentity){
            throw new \Exception();
        }
		
        // look for existing physical entities with the provided fiscal code
		$physicalEntity = $this->physicalEntityRepository->findOneByFiscalCode($fiscalCode);

        // if exists
        if ($physicalEntity) {
            // and is already a user and has the provided identity
            if (isset($physicalEntity->Identities->$providerKey)) {
                throw new \Exception();
            }
            
            $identity->$providerKey->VerificationCode = substr(bin2hex(random_bytes(5)),0,4);          
        } else {
            // physical entity creation
            $physicalEntity = new stdClass();
            $physicalEntity->FirstName = $firstName;
            $physicalEntity->LastName = $lastName;
            $physicalEntity->FiscalCode = $fiscalCode;
            $physicalEntity->UserDBRequested = $createUserDB;
            
			$contacts = new stdClass();
			$contacts->Email = new stdClass();
			$contacts->Email->Value = $email;
			$contacts->Email->Verified = $identityProvider->isEmailVerified();
			if(!$contacts->Email->Verified){
				$contacts->Email->VerificationCode = substr(bin2hex(random_bytes(5)),0,4);
			}
            
			$physicalEntity->Contacts = $contacts;         
        }
        
        $identityProvider->finalize($identity);
        
        // add identity
		$this->addIdentity($physicalEntity, $identity);
        
        // aggiungiamo la rappresentanza verso se stesso
        // $this->addUserLegalEntity($PhisicalEntity);
        
        $this->physicalEntityRepository->save($physicalEntity);
        
        // aggiunta db, se necessario
        if ($createUserDB) {
            // $this->createDatabaseUserInfo($physicalEntity);
        }
		
		return $this->_reduceDocument($physicalEntity);
    } 
    
    private function addIdentity($physicalEntity, $identity) {     
        if (!isset($physicalEntity->Identities)) {
            $physicalEntity->Identities = new \stdClass();
        }
        
        foreach ($identity as $key => $value) {
            $physicalEntity->Identities->$key = $value;
            $physicalEntity->Identities->$key->Verified = false;
        }
    }    
    
    private function getProvider($identity) {
        if (!is_object($identity)) {
            throw new \Exception();
        }
        
        foreach ($this->identityProviders as $identityProvider) {
            if ($identityProvider->supports($identity)) {
                return $identityProvider;
            }
        }
        
        throw new \Exception();
    }
    
	private function _reduceDocument($doc) {
    	$ret = new stdClass();
		$ret->_id = $doc->_id;
        $ret->CreationDate = $doc->CreationDate;
        $ret->UpdateDate = $doc->UpdateDate;
        
		$ret->FirstName = $doc->FirstName;
		$ret->LastName = $doc->LastName;
		$ret->FiscalCode = $doc->FiscalCode;       
		$ret->Contacts = $doc->Contacts;
        
        if (isset($doc->UserDB) && isset($doc->UserPassword)) {
            $ret->UserDB = $doc->UserDB;
            $ret->UserPassword = $doc->UserPassword;
        }
        
        if (isset($doc->Roles)) {
            $ret->Roles = $doc->Roles;
        }
		
		if (isset($doc->Devices)) {
            $ret->Devices = $doc->Devices;
        }
        
		return $ret;
    }    
}

==> schema31-safe-hub-lib-0.1.3-361ab4.zip: src/Application/Service/User.php <==
<?php

namespace Schema31\SafeHubLib\Application\Service;

use Schema31\SafeHubLib\Infrastructure\IdentityProvider\SafeIdentityProvider;
use Schema31\SafeHubLib\Infrastructure\IdentityProvider\GoogleIdentityProvider;
use Schema31\SafeHubLib\Infrastructure\IdentityProvider\FacebookIdentityProvider;

use Schema31\SafeHubLib\Domain\Repository\PhysicalEntityRepositoryInterface;

/**
 * Description of User
 *
 * @author Antonio Turdo <aturdo@schema31.it>
 */
class User {
    
    private $identityProviders;
    private $physicalEntityRepository;
    
    public function __construct(PhysicalEntityRepositoryInterface $physicalEntityRepository) {
        $this->identityProviders = [
            new SafeIdentityProvider(),
            new GoogleIdentityProvider(),
            new FacebookIdentityProvider()
        ];
        
        $this->physicalEntityRepository = $physicalEntityRepository;
    }
    
    public function createUser(string $firstName, string $lastName, string $fiscalCode, string $email, \stdClass $identity, bool $createUserDB = false) {
		$identityProvider = $this->getProvider($identity);

        // validate the identity
        $identityProvider->validate($identity);        
        
        // normalize the identity
        $identityProvider->normalize($identity);
        
        $providerKey = $identityProvider->getProviderKey();
		
        // look for existing physical entities with the provided identity
        $userByIdentity = $this->physicalEntityRepository->findOneByIdentity($providerKey, $identity->$providerKey->Loginkey);
        
        if($userByIdentity){
            throw new \Exception();
        }
		
        // look for existing physical entities with the provided fiscal code
		$physicalEntity = $this->physicalEntityRepository->findOneByFiscalCode($fiscalCode);

        // if exists
        if ($physicalEntity) {
            // and is already a user and has the provided identity
            if (isset($physicalEntity->Identities->$providerKey)) {
                throw new \Exception();
            }
            
            $identity->$providerKey->VerificationCode = substr(bin2hex(random_bytes(5)),0,4);          
        } else {
            // physical entity creation
            $physicalEntity = new stdClass();
            $physicalEntity->FirstName = $firstName;
            $physicalEntity->LastName = $lastName;
            $physicalEntity->FiscalCode = $fiscalCode;
            $physicalEntity->UserDBRequested = $createUserDB;
            
			$contacts = new stdClass();
			$contacts->Email = new stdClass();
			$contacts->Email->Value = $email;
			$contacts->Email->Verified = $identityProvider->isEmailVerified();
			if(!$contacts->Email->Verified){
				$contacts->Email->VerificationCode = substr(bin2hex(random_bytes(5)),0,4);
			}
            
			$physicalEntity->Contacts = $contacts;         
        }
        
        $identityProvider->finalize($identity);
        
        // add identity
		$this->addIdentity($physicalEntity, $identity);
        
        // aggiungiamo la rappresentanza verso se stesso
        // $this->addUserLegalEntity($PhisicalEntity);
        
        $this->physicalEntityRepository->save($physicalEntity);
        
        // aggiunta db, se necessario
        if ($createUserDB) {
            // $this->createDatabaseUserInfo($physicalEntity);
        }
		
		return $this->_reduceDocument($physicalEntity);
    } 
    
    private function addIdentity($physicalEntity, $identity) {     
        if (!isset($physicalEntity->Identities)) {
            $physicalEntity->Identities = new \stdClass();
        }
        
        foreach ($identity as $key => $value) {
            $physicalEntity->Identities->$key = $value;
            $physicalEntity->Identities->$key->Verified = false;
        }
    }    
    
    private function getProvider($identity) {
        if (!is_object($identity)) {
            throw new \Exception();
        }
        
        foreach ($this->identityProviders as $identityProvider) {
            if ($identityProvider->supports($identity)) {
                return $identityProvider;
            }
        }
        
        throw new \Exception();
    }
    
	private function _reduceDocument($doc) {
    	$ret = new stdClass();
		$ret->_id = $doc->_id;
        $ret->CreationDate = $doc->CreationDate;
        $ret->UpdateDate = $doc->UpdateDate;
        
		$ret->FirstName = $doc->FirstName;
		$ret->LastName = $doc->LastName;
		$ret->FiscalCode = $doc->FiscalCode;       
		$ret->Contacts = $doc->Contacts;
        
        if (isset($doc->UserDB) && isset($doc->UserPassword)) {
            $ret->UserDB = $doc->UserDB;
            $ret->UserPassword = $doc->UserPassword;
        }
        
        if (isset($doc->Roles)) {
            $ret->Roles = $doc->Roles;
        }
		
		if (isset($doc->Devices)) {
            $ret->Devices = $doc->Devices;
        }
        
		return $ret;
    }    
}
